import { MatchEntity } from '../../../domain/match.entity';
import { Candidate } from '../../types/algorithm.types';
import { MatchQuery } from './match.query';
import { AdRepositoryPort } from '@modules/ad/core/application/ports/ad.repository.port';

export abstract class Algorithm {
  protected candidates: Candidate[];
  protected selector: Selector;
  protected processors: Processor[];
  constructor(
    protected readonly query: MatchQuery,
    protected readonly repository: AdRepositoryPort,
  ) {}

  /**
   * Return Ads that matches the query as Matches
   */
  match = async (): Promise<MatchEntity[]> => {
    this.candidates = await this.selector.select();
    for (const processor of this.processors) {
      this.candidates = await processor.execute(this.candidates);
    }
    return this.candidates.map((candidate: Candidate) =>
      MatchEntity.create({ adId: candidate.ad.id }),
    );
  };
}

/**
 * A selector queries potential candidates in a repository
 */
export abstract class Selector {
  protected readonly query: MatchQuery;
  protected readonly repository: AdRepositoryPort;
  constructor(query: MatchQuery, repository: AdRepositoryPort) {
    this.query = query;
    this.repository = repository;
  }
  abstract select(): Promise<Candidate[]>;
}

/**
 * A processor processes candidates information
 */
export abstract class Processor {
  protected readonly query: MatchQuery;
  constructor(query: MatchQuery) {
    this.query = query;
  }
  abstract execute(candidates: Candidate[]): Promise<Candidate[]>;
}
