import {
  AD_REPOSITORY,
  AD_CARPOOL_ROUTE_PROVIDER,
  INPUT_DATETIME_TRANSFORMER,
  PARAMS_PROVIDER,
} from '@modules/ad/ad.di-tokens';
import { DateTimeTransformerPort } from '@modules/ad/core/application/ports/datetime-transformer.port';
import { DefaultParamsProviderPort } from '@modules/ad/core/application/ports/default-params-provider.port';
import { CarpoolRouteProviderPort } from '@modules/ad/core/application/ports/carpool-route-provider.port';
import { MatchQuery } from '@modules/ad/core/application/queries/match/match.query';
import { MatchQueryHandler } from '@modules/ad/core/application/queries/match/match.query-handler';
import { AlgorithmType } from '@modules/ad/core/application/types/algorithm.types';
import { Waypoint } from '@modules/ad/core/application/types/waypoint.type';
import { Frequency, Role } from '@modules/ad/core/domain/ad.types';
import { MatchEntity } from '@modules/ad/core/domain/match.entity';
import { Test, TestingModule } from '@nestjs/testing';

const originWaypoint: Waypoint = {
  position: 0,
  lat: 48.689445,
  lon: 6.17651,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const destinationWaypoint: Waypoint = {
  position: 1,
  lat: 48.8566,
  lon: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};

const mockAdRepository = {
  getCandidates: jest.fn().mockImplementation(() => [
    {
      ad: {
        id: 'cc260669-1c6d-441f-80a5-19cd59afb777',
      },
      role: Role.DRIVER,
    },
  ]),
};

const mockDefaultParamsProvider: DefaultParamsProviderPort = {
  getParams: () => {
    return {
      DEPARTURE_TIME_MARGIN: 900,
      DRIVER: false,
      SEATS_PROPOSED: 3,
      PASSENGER: true,
      SEATS_REQUESTED: 1,
      STRICT: false,
      TIMEZONE: 'Europe/Paris',
      ALGORITHM_TYPE: AlgorithmType.PASSENGER_ORIENTED,
      REMOTENESS: 15000,
      USE_PROPORTION: true,
      PROPORTION: 0.3,
      USE_AZIMUTH: true,
      AZIMUTH_MARGIN: 10,
      MAX_DETOUR_DISTANCE_RATIO: 0.3,
      MAX_DETOUR_DURATION_RATIO: 0.3,
    };
  },
};

const mockInputDateTimeTransformer: DateTimeTransformerPort = {
  fromDate: jest.fn(),
  toDate: jest.fn(),
  day: jest.fn(),
  time: jest.fn(),
};

const mockRouteProvider: CarpoolRouteProviderPort = {
  getBasic: jest.fn(),
};

describe('Match Query Handler', () => {
  let matchQueryHandler: MatchQueryHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        MatchQueryHandler,
        {
          provide: AD_REPOSITORY,
          useValue: mockAdRepository,
        },
        {
          provide: PARAMS_PROVIDER,
          useValue: mockDefaultParamsProvider,
        },
        {
          provide: INPUT_DATETIME_TRANSFORMER,
          useValue: mockInputDateTimeTransformer,
        },
        {
          provide: AD_CARPOOL_ROUTE_PROVIDER,
          useValue: mockRouteProvider,
        },
      ],
    }).compile();

    matchQueryHandler = module.get<MatchQueryHandler>(MatchQueryHandler);
  });

  it('should be defined', () => {
    expect(matchQueryHandler).toBeDefined();
  });

  it('should return a Match entity', async () => {
    const matchQuery = new MatchQuery({
      algorithmType: AlgorithmType.PASSENGER_ORIENTED,
      driver: false,
      passenger: true,
      frequency: Frequency.PUNCTUAL,
      fromDate: '2023-08-28',
      toDate: '2023-08-28',
      schedule: [
        {
          time: '07:05',
          day: 1,
          margin: 900,
        },
      ],
      strict: false,
      waypoints: [originWaypoint, destinationWaypoint],
    });
    const matches: MatchEntity[] = await matchQueryHandler.execute(matchQuery);
    expect(matches.length).toBeGreaterThanOrEqual(0);
  });
});
