import { AggregateRoot, AggregateID } from '@mobicoop/ddd-library';
import { CreateRouteProps, RouteProps, Route } from './route.types';
import { v4 } from 'uuid';
import { RouteNotFoundException } from './route.errors';

export class RouteEntity extends AggregateRoot<RouteProps> {
  protected readonly _id: AggregateID;

  static create = async (create: CreateRouteProps): Promise<RouteEntity> => {
    const route: Route = await create.georouter.route(
      create.waypoints,
      create.georouterSettings,
    );
    if (!route) throw new RouteNotFoundException();
    const routeProps: RouteProps = {
      distance: route.distance,
      duration: route.duration,
      fwdAzimuth: route.fwdAzimuth,
      backAzimuth: route.backAzimuth,
      distanceAzimuth: route.distanceAzimuth,
      points: route.points,
    };
    return new RouteEntity({
      id: v4(),
      props: routeProps,
    });
  };

  validate(): void {
    // entity business rules validation to protect it's invariant before saving entity to a database
  }

  // private static getPaths = (
  //   roles: Role[],
  //   waypoints: WaypointProps[],
  // ): Path[] => {
  //   const paths: Path[] = [];
  //   if (roles.includes(Role.DRIVER) && roles.includes(Role.PASSENGER)) {
  //     if (waypoints.length == 2) {
  //       // 2 points => same route for driver and passenger
  //       paths.push(this.createGenericPath(waypoints));
  //     } else {
  //       paths.push(
  //         this.createDriverPath(waypoints),
  //         this.createPassengerPath(waypoints),
  //       );
  //     }
  //   } else if (roles.includes(Role.DRIVER)) {
  //     paths.push(this.createDriverPath(waypoints));
  //   } else if (roles.includes(Role.PASSENGER)) {
  //     paths.push(this.createPassengerPath(waypoints));
  //   }
  //   return paths;
  // };

  // private static createGenericPath = (waypoints: WaypointProps[]): Path =>
  //   this.createPath(waypoints, PathType.GENERIC);

  // private static createDriverPath = (waypoints: WaypointProps[]): Path =>
  //   this.createPath(waypoints, PathType.DRIVER);

  // private static createPassengerPath = (waypoints: WaypointProps[]): Path =>
  //   this.createPath(
  //     [waypoints[0], waypoints[waypoints.length - 1]],
  //     PathType.PASSENGER,
  //   );

  // private static createPath = (
  //   points: WaypointProps[],
  //   type: PathType,
  // ): Path => ({
  //   type,
  //   points,
  // });
}
