import { GeorouterPort } from '@modules/geography/core/application/ports/georouter.port';
import { RouteEntity } from '@modules/geography/core/domain/route.entity';
import { RouteNotFoundException } from '@modules/geography/core/domain/route.errors';
import {
  Point,
  CreateRouteProps,
} from '@modules/geography/core/domain/route.types';

const originPoint: Point = {
  lat: 48.689445,
  lon: 6.17651,
};
const destinationPoint: Point = {
  lat: 48.8566,
  lon: 2.3522,
};

const mockGeorouter: GeorouterPort = {
  route: jest
    .fn()
    .mockImplementationOnce(() => ({
      distance: 350101,
      duration: 14422,
      fwdAzimuth: 273,
      backAzimuth: 93,
      distanceAzimuth: 336544,
      points: [
        {
          lon: 6.1765102,
          lat: 48.689445,
        },
        {
          lon: 4.984578,
          lat: 48.725687,
        },
        {
          lon: 2.3522,
          lat: 48.8566,
        },
      ],
      steps: [],
    }))
    .mockImplementationOnce(() => []),
};

const createRouteProps: CreateRouteProps = {
  waypoints: [
    {
      position: 0,
      ...originPoint,
    },
    {
      position: 1,
      ...destinationPoint,
    },
  ],
  georouter: mockGeorouter,
  georouterSettings: {
    points: true,
    detailedDistance: false,
    detailedDuration: false,
  },
};

describe('Route entity create', () => {
  it('should create a new entity', async () => {
    const route: RouteEntity = await RouteEntity.create(createRouteProps);
    expect(route.id.length).toBe(36);
    expect(route.getProps().duration).toBe(14422);
  });

  it('should throw an exception if route is not found', async () => {
    try {
      await RouteEntity.create(createRouteProps);
    } catch (e: any) {
      expect(e).toBeInstanceOf(RouteNotFoundException);
    }
  });
});
