import { Module, Provider } from '@nestjs/common';
import { MESSAGE_PUBLISHER } from './messager.di-tokens';
import {
  MessageBrokerModule,
  MessageBrokerModuleOptions,
  MessageBrokerPublisher,
} from '@mobicoop/message-broker-module';
import { ConfigModule, ConfigService } from '@nestjs/config';

const imports = [
  MessageBrokerModule.forRootAsync({
    imports: [ConfigModule],
    inject: [ConfigService],
    useFactory: async (
      configService: ConfigService,
    ): Promise<MessageBrokerModuleOptions> => ({
      uri: configService.get<string>('MESSAGE_BROKER_URI') as string,
      exchange: configService.get<string>('MESSAGE_BROKER_EXCHANGE') as string,
      name: 'matcher',
      handlers: {
        adCreated: {
          routingKey: 'ad.created',
          queue: 'matcher-ad-created',
        },
        adUpdated: {
          routingKey: 'ad.updated',
          queue: 'matcher-ad-updated',
        },
        adDeleted: {
          routingKey: 'ad.deleted',
          queue: 'matcher-ad-deleted',
        },
      },
    }),
  }),
];

const providers: Provider[] = [
  {
    provide: MESSAGE_PUBLISHER,
    useClass: MessageBrokerPublisher,
  },
];

@Module({
  imports,
  providers,
  exports: [MESSAGE_PUBLISHER],
})
export class MessagerModule {}
